#
# Autogenerated by Thrift Compiler (0.10.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
import sys

from thrift.transport import TTransport


class Token(object):
    """
    Attributes:
     - word
     - tag
     - lemma
    """

    thrift_spec = (
        None,  # 0
        (1, TType.STRING, 'word', 'UTF8', None, ),  # 1
        (2, TType.STRING, 'tag', 'UTF8', None, ),  # 2
        (3, TType.STRING, 'lemma', 'UTF8', None, ),  # 3
    )

    def __init__(self, word=None, tag=None, lemma=None,):
        self.word = word
        self.tag = tag
        self.lemma = lemma

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, (self.__class__, self.thrift_spec))
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.word = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.tag = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.lemma = iprot.readString().decode('utf-8') if sys.version_info[0] == 2 else iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, (self.__class__, self.thrift_spec)))
            return
        oprot.writeStructBegin('Token')
        if self.word is not None:
            oprot.writeFieldBegin('word', TType.STRING, 1)
            oprot.writeString(self.word.encode('utf-8') if sys.version_info[0] == 2 else self.word)
            oprot.writeFieldEnd()
        if self.tag is not None:
            oprot.writeFieldBegin('tag', TType.STRING, 2)
            oprot.writeString(self.tag.encode('utf-8') if sys.version_info[0] == 2 else self.tag)
            oprot.writeFieldEnd()
        if self.lemma is not None:
            oprot.writeFieldBegin('lemma', TType.STRING, 3)
            oprot.writeString(self.lemma.encode('utf-8') if sys.version_info[0] == 2 else self.lemma)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)



#!/usr/bin/env python
#
# Autogenerated by Thrift Compiler (0.9.3)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:utf8strings
#

import sys
import pprint
from urlparse import urlparse
from thrift.transport import TTransport
from thrift.transport import TSocket
from thrift.transport import TSSLSocket
from thrift.transport import THttpClient
from thrift.protocol import TBinaryProtocol

from spacyThrift import SpacyThrift
from spacyThrift.ttypes import *

if len(sys.argv) <= 1 or sys.argv[1] == '--help':
  print('')
  print('Usage: ' + sys.argv[0] + ' [-h host[:port]] [-u url] [-f[ramed]] [-s[sl]] function [arg1 [arg2...]]')
  print('')
  print('Functions:')
  print('   tag(string sentence)')
  print('')
  sys.exit(0)

pp = pprint.PrettyPrinter(indent = 2)
host = 'localhost'
port = 9090
uri = ''
framed = False
ssl = False
http = False
argi = 1

if sys.argv[argi] == '-h':
  parts = sys.argv[argi+1].split(':')
  host = parts[0]
  if len(parts) > 1:
    port = int(parts[1])
  argi += 2

if sys.argv[argi] == '-u':
  url = urlparse(sys.argv[argi+1])
  parts = url[1].split(':')
  host = parts[0]
  if len(parts) > 1:
    port = int(parts[1])
  else:
    port = 80
  uri = url[2]
  if url[4]:
    uri += '?%s' % url[4]
  http = True
  argi += 2

if sys.argv[argi] == '-f' or sys.argv[argi] == '-framed':
  framed = True
  argi += 1

if sys.argv[argi] == '-s' or sys.argv[argi] == '-ssl':
  ssl = True
  argi += 1

cmd = sys.argv[argi]
args = sys.argv[argi+1:]

if http:
  transport = THttpClient.THttpClient(host, port, uri)
else:
  socket = TSSLSocket.TSSLSocket(host, port, validate=False) if ssl else TSocket.TSocket(host, port)
  if framed:
    transport = TTransport.TFramedTransport(socket)
  else:
    transport = TTransport.TBufferedTransport(socket)
protocol = TBinaryProtocol.TBinaryProtocol(transport)
client = SpacyThrift.Client(protocol)
transport.open()

if cmd == 'tag':
  if len(args) != 1:
    print('tag requires 1 args')
    sys.exit(1)
  pp.pprint(client.tag(args[0],))

else:
  print('Unrecognized method %s' % cmd)
  sys.exit(1)

transport.close()



import logging
import sys
from typing import List

import click
import coloredlogs
import spacy
from spacy.language import Language
from spacy.tokens.doc import Doc
from spacy.tokens.token import Token as SpacyToken

from spacyThrift import SpacyThrift
from spacyThrift.ttypes import Token

from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol
from thrift.server import TServer


class Handler:
    def __init__(self, nlp: Language) -> None:
        self.nlp = nlp

    @classmethod
    def _lemma(cls, token: SpacyToken):
        if token.lemma_ != "-PRON-":
            return token.lemma_.lower().strip()
        else:
            return token.lower_

    def tag(self, sentence: str) -> List[Token]:
        document = self.nlp(sentence, parse=False, entity=False)   # type: Doc
        return [
            Token(element.orth_, element.tag_, self._lemma(element))
            for element in document   # type: SpacyToken
        ]


@click.command()
@click.option('--port', default=9090)
@click.option('--language', default='en')
def serve(port: int, language: str) -> None:
    coloredlogs.install(stream=sys.stderr, level=logging.INFO,
                        fmt='%(asctime)s %(name)s %(levelname)s %(message)s')
    logging.info("Loading ...")
    nlp = spacy.load(language, parser=False, entity=False,
                     create_pipeline=lambda nlp: [nlp.tagger])  # type: Language
    handler = Handler(nlp)
    processor = SpacyThrift.Processor(handler)
    server_socket = TSocket.TServerSocket(port=port)
    transport_factory = TTransport.TBufferedTransportFactory()
    protocol_factory = TBinaryProtocol.TBinaryProtocolFactory()
    server = TServer.TThreadedServer(processor, server_socket, transport_factory, protocol_factory)
    logging.info("Serving on port %d ...", port)
    server.serve()


if __name__ == '__main__':
    serve()




#!/usr/bin/env python

from setuptools import setup

setup(name='spacy-thrift',
      version='0.2',
      description='spaCy-as-a-service using Thrift',
      keywords='natural language processing',
      url='https://github.com/turbolent/spacy-thrift',
      author='Bastian Mueller',
      author_email='bastian@turbolent.com',
      classifiers=[
          "License :: OSI Approved :: MIT License",
          "Topic :: Text Processing",
      ],
      packages=['spacyThrift'],
      install_requires=["thrift", "spacy"])
